"""
Ceres plusplus is a module designed to extend the functionality of ceres
(https://github.com/rabrahm/ceres) in order to calculate activity indicators
from spectra reduced with the tool, and to easily extract the indicators that
ceres does calculate, for example the CCF FWHM and the BIS.
"""

from .cerespp import get_activities
from .cpplots import ccf_gauss_plot
from .cpplots import ccf_plot
from .cpplots import line_plot
from .cpplots import line_plot_from_file
from .crosscorr import ccf
from .crosscorr import ccf_fit
from .spectra_utils import median_combine
from .spectra_utils import median_combine_1d
from .spectra_utils import merge_echelle
from .spectra_utils import velocity_correction
from .spectra_utils import correct_to_rest



"""constants.py
This module contains constants to be used throughout the code.
I'm storing them here to remove clutter from the main code.
"""

from pkg_resources import resource_filename


masksdir = resource_filename('cerespp', 'masks')

# relevant lines
CaH = 3968.47
CaK = 3933.664
CaV = 3901
CaR = 4001
Ha = 6562.808
HeI = 5875.62
NaID1 = 5895.92
NaID2 = 5889.95

# Plotting
fontsize = 22
fontname = 'serif'
tick_labelsize = 18



"""cerespp.

Main routine for ceres plusplus.

Author: Jose Vines
"""
import numpy as np
from astropy.io import fits
from tqdm import tqdm

from .spectra_utils import correct_to_rest, merge_echelle
from .utils import get_line_flux
from .constants import *


def get_activities(files, out, mask='G2', save=False):
    """Main function.

    Parameters
    ----------
    files: array_like
        An array or list with the spectra filenames.
    out: str
        The output directory for the results.
    save: bool, optional
        Set to True to save the merged echelle spectra. Default is False.

    Returns
    -------
    data: array_like
        An array with the activity indices.
    header: array_like
        An array with the activity indices identifiers.

    """
    nofwhm = False

    # Activity holders
    bjd = []
    s_indices = []
    s_err = []
    ha = []
    ha_err = []
    hei = []
    hei_err = []
    nai = []
    nai_err = []
    bis = []
    bis_err = []
    fwhm = []
    fwhm_err = []
    contrast = []

    for fname in tqdm(files):
        hdul = fits.open(fname)
        data = hdul[0].data

        bjd.append(hdul[0].header['BJD_OUT'])  # Extract BJD
        bis.append(hdul[0].header['BS'])  # Extract BIS
        bis_err.append(hdul[0].header['BS_E'])  # Extract BIS error
        try:
            fwhm.append(hdul[0].header['FWHM'])  # Try and get the FWHM
            # This calculates the error on the FWHM, inspired by the RV error
            # Calculation from the ceres paper.
            fwhm_err.append(hdul[0].header['DISP'] / hdul[0].header['SNR'])
        except KeyError:  # if the FWHM is not in the header for some reason..
            nofwhm = True
        contrast.append(hdul[0].header['XC_MIN'])

        w, f = correct_to_rest(data, mask=mask)

        prod = np.stack((w, f, data[4, :, :], data[8, :, :]))

        waves, fluxes, errors, sn = merge_echelle(prod, hdul[0].header,
                                                  out=out, save=save)

        # Get S index
        # First is CaV
        NV, sNV = get_line_flux(waves, fluxes, CaV, 20, filt='square',
                                error=errors)

        # Now CaR
        NR, sNR = get_line_flux(waves, fluxes, CaR, 20,
                                filt='square', error=errors)

        # Now CaK
        NK, sNK = get_line_flux(waves, fluxes, CaK, 1.09,
                                filt='triangle', error=errors)

        # Now CaH
        NH, sNH = get_line_flux(waves, fluxes, CaH, 1.09,
                                filt='triangle', error=errors)

        S = (NH + NK) / (NR + NV)
        sSnum = np.sqrt(sNH ** 2 + sNK ** 2)
        sSden = np.sqrt(sNV ** 2 + sNR ** 2)
        sS = np.sqrt((sSnum / (NH + NK)) ** 2 + (sSden / (NR + NV)) ** 2)

        s_indices.append(S)
        s_err.append(sS)

        # Now Halpha
        FHa, sFHa = get_line_flux(waves, fluxes, Ha, 0.678,
                                  filt='square', error=errors)

        # Now F1
        F1, sF1 = get_line_flux(waves, fluxes, 6550.87,
                                10.75, filt='square', error=errors)

        # Now F2
        F2, sF2 = get_line_flux(waves, fluxes, 6580.309,
                                8.75, filt='square', error=errors)

        Halpha = FHa / (0.5 * (F1 + F2))
        sden = np.sqrt(sF1 ** 2 + sF2 ** 2)
        sHalpha = np.sqrt((sFHa / FHa) ** 2 + (sden / (F1 + F2)) ** 2)

        ha.append(Halpha)
        ha_err.append(sHalpha)

        # Now HeI
        FHeI, sFHeI = get_line_flux(waves, fluxes, HeI, 0.2,
                                    filt='square', error=errors)

        # Now F1
        F1, sF1 = get_line_flux(waves, fluxes, 5874.5,
                                0.5, filt='square', error=errors)

        # Now F2
        F2, sF2 = get_line_flux(waves, fluxes, 5879,
                                0.5, filt='square', error=errors)

        HelI = FHeI / (0.5 * (F1 + F2))
        sden = np.sqrt(sF1 ** 2 + sF2 ** 2)
        sHelI = np.sqrt((sFHeI / FHeI) ** 2 + (sden / (F1 + F2)) ** 2)

        hei.append(HelI)
        hei_err.append(sHelI)

        # Now NaI D1 D2
        D1, sD1 = get_line_flux(waves, fluxes, NaID1, 1,
                                filt='square', error=errors)

        D2, sD2 = get_line_flux(waves, fluxes, NaID2, 1,
                                filt='square', error=errors)

        L, sL = get_line_flux(waves, fluxes, 5805, 10, filt='square',
                              error=errors)

        R, sR = get_line_flux(waves, fluxes, 6090, 20, filt='square',
                              error=errors)

        NaID1D2 = (D1 + D2) / (R + L)
        sNanum = np.sqrt(sD1 ** 2 + sD2 ** 2)
        sNaden = np.sqrt(sL ** 2 + sR ** 2)
        sNaID1D2 = np.sqrt((sNanum / (D1 + D2)) ** 2 + (sNaden / (R + L)) ** 2)

        nai.append(NaID1D2)
        nai_err.append(sNaID1D2)

        if not nofwhm:
            header = 'bjd S e_S Halpha e_Halpha HeI'
            header += ' e_HeI NaID1D2 e_NaID1D2 BIS e_BIS FWHM e_FWHM CONTRAST'

            data = np.vstack(
                [
                    bjd,
                    s_indices, s_err,
                    ha, ha_err,
                    hei, hei_err,
                    nai, nai_err,
                    bis, bis_err,
                    fwhm, fwhm_err,
                    contrast
                ]
            ).T
        else:
            header = 'bjd S e_S Halpha e_Halpha HeI'
            header += ' e_HeI NaID1D2 e_NaID1D2 BIS e_BIS CONTRAST'

            data = np.vstack(
                [
                    bjd,
                    s_indices, s_err,
                    ha, ha_err,
                    hei, hei_err,
                    nai, nai_err,
                    bis, bis_err,
                    contrast
                ]
            ).T
        np.savetxt(
            f'{out}.dat', data, header=header)
    return data, header


import numpy as np
from scipy.integrate import trapz
from scipy.interpolate import interp1d
from scipy.signal import triang
from scipy.stats import norm


def gauss_fit(r, a, mu, sig, off):
    """Wrapper for a gaussian function used in a fitting routine."""
    return off + a * norm.pdf(r, loc=mu, scale=sig)


def get_ini_end(x, mid, width):
    """Get initial and ending indices for a band with a given width."""
    ini = np.argmin(abs(x - (mid - width)))
    end = np.argmin(abs(x - (mid + width)))
    return ini, end


def get_response(line, width, filt='square'):
    """Instantiate a response filter and return filter width."""
    if filt == 'square':
        width /= 2
        w_resp = np.linspace(line - width, line + width, 1000)
        resp_f = interp1d(w_resp, np.ones(
            1000), bounds_error=False, fill_value=0)
    elif filt == 'triangle':
        tri = triang(999, True)
        w_resp = np.linspace(line - width, line + width, 999)
        resp_f = interp1d(w_resp, tri, bounds_error=False, fill_value=0)
    return resp_f, width


def get_line_flux(wavelength, flux, line, width, filt='square', error=None):
    """Calculate line flux with a given response filter.

    Accepted filter values are square or triangle, if triangle
    then the width will be taken as the FWHM of the filter bandpass.

    Also calculates error in flux if an error array is provided.

    To calculate the flux first a filter response function is created
    Then the flux is rebbined in the line width, convolved with the
    response filter and finally normalized by the integrated response
    function.
    """
    resp_f, width = get_response(line, width, filt)
    ini, end = get_ini_end(wavelength, line, width)
    w = np.linspace(line - width, line + width, (end - ini))
    intp = interp1d(wavelength[ini - 2:end + 2], flux[ini - 2:end + 2])
    intp_flux = intp(w)
    response = resp_f(w)
    integrated_flux = trapz(intp_flux * response, w) / trapz(response, w)
    if error is not None:
        num = (error[ini:end] * resp_f(wavelength[ini:end])) ** 2
        den = resp_f(wavelength[ini:end]) ** 2
        var = num.sum() / den.sum()
        sigma = np.sqrt(var)
        return integrated_flux, sigma
    return integrated_flux



"""Spectra utils.

Various utilities for spectra.

Author: Jose Vines
"""
import copy
import os

import astropy.constants as const
import numpy as np
from astropy.io import fits
from scipy.interpolate import interp1d
from tqdm import tqdm

from .crosscorr import ccf, ccf_fit


def feros_velocity_correction(data, rv, create_fits=False, header=None, out=''):
    """Radial velocity correction for a FEROS spectrum.

    Parameters
    ----------
    data : array_like
        An array shaped (2, ord, N) where the first entry are the wavelengths,
        the second is the deblazed continuum normalized flux.
        Ord is the number of echelle orders and N the number of data

    rv : float
        The radial velocity in m/s

    create_fits : bool, optional
        True to save a fits file with the result.

    out : str, optional
        The location where to save the rest_frame spectra.

    Returns
    -------
    wavelength : array_like
        An array with the rest frame wavelengths.

    flux : array_like
        An array with the corresponding fluxes.

    """
    # Read fits file
    #     hdul = fits.open(spec)
    # Extract RV
    #     if not rv:
    #         rv = hdul[0].header['RV'] * u.km / u.s
    #         rv = rv.to(u.m / u. s)
    # Create gamma
    beta = rv / const.c
    gamma = 1 + beta.value
    # Extract wavelength per order
    wave = data[0, :, :]
    # Extract flux per order
    flux = data[1, :, :]
    # Extract the error of each order
    error = data[2, :, :]
    # Extract the snr of each order
    snr = data[3, :, :]
    orders = wave.shape[0]
    wave_rest = copy.deepcopy(wave)
    # Move spectra to rest frame
    for o in range(orders):
        wave_rest[o, :] /= gamma
    if create_fits:
        # Create new fits file
        date = header['HIERARCH SHUTTER START DATE'].split('-')
        ut = header['HIERARCH SHUTTER START UT'].split(':')
        out += header['HIERARCH TARGET NAME'] + '_'
        for d in date:
            out += d
        out += '_UT'
        for u in ut:
            out += u
        out += '_rest_frame.fits'
        hdu = fits.PrimaryHDU(np.stack((wave_rest, flux, error, snr)),
                              header=header)
        try:
            hdu.writeto(out)
        except OSError:
            os.remove(out)
            hdu.writeto(out)
    return wave_rest, flux


def velocity_correction(instrument, spec_list, rvs=[]):
    """Radial velocity correction for a list of spectra.

    Parameters
    ----------
    instrument : str
        The instrument with which the spectra was obtained.
    spec_list : array_like
        An array with filenames pointing to the fits files.

    """
    if instrument.lower() == 'feros':
        corrector = feros_velocity_correction
    if not rvs:
        for spec in tqdm(spec_list, desc='Spectra'):
            corrector(spec, True)
    else:
        for spec, rv in tqdm(zip(spec_list, rvs),
                             desc='Spectra', total=len(spec_list)):
            corrector(spec, True, rv)
    pass


def median_combine(spec_list, nord=None, plx=None):
    """Median combine rest frame spectra.

    Parameters
    ----------
    spec_list : array_like
        Array with spectra files.
    nord : int, optional
        Number of echelle orders.
    plx : float, optional
        Target's parallax.

    """
    wavelengths = []
    fluxes = []
    header = fits.open(spec_list[0])[0].header
    targ_name = header['HIERARCH TARGET NAME']
    ra = header['HIERARCH RA']
    dec = header['HIERARCH DEC']
    inst = header['INST']
    if nord is None:
        nord = header['NAXIS2']
    for o in tqdm(range(nord), desc='Order'):
        combiner = []
        for spec in spec_list:
            hdul = fits.open(spec)
            flux = hdul[0].data[1, o, :]
            wave = hdul[0].data[0, o, :]
            combiner.append(flux)
            hdul.close()
        combiner = np.array(combiner)
        combined = np.median(combiner, axis=0)
        fluxes.append(combined)
        wavelengths.append(wave)
    final_waves = np.vstack(wavelengths)
    final_fluxes = np.vstack(fluxes)
    final_out = np.stack([final_waves, final_fluxes])
    out = targ_name + '_stacked.fits'
    hdr = fits.Header()
    hdr['NAME'] = targ_name
    hdr['PLX'] = plx
    hdr['RA (deg)'] = ra
    hdr['DEC (deg)'] = dec
    hdr['INST'] = inst
    hdu = fits.PrimaryHDU(final_out, header=hdr)
    try:
        hdu.writeto(out)
    except OSError:
        os.remove(out)
        hdu.writeto(out)
    pass


def median_combine_1d(spec_list, plx=None):
    """Median combine 1d rest frame spectra.

    Parameters
    ----------
    spec_list : array_like
        Array with spectra files.
    plx : float, optional
        Target's parallax.

    """
    combiner = []
    header = fits.open(spec_list[0])[0].header
    wave = fits.open(spec_list[0])[0].data[0, :]
    new_w = np.linspace(wave[0], wave[-1], len(wave))
    targ_name = header['HIERARCH TARGET NAME']
    ra = header['HIERARCH RA (deg)']
    dec = header['HIERARCH DEC (deg)']
    inst = header['INST']
    for spec in spec_list:
        hdul = fits.open(spec)
        flux = hdul[0].data[1, :]
        wave = hdul[0].data[0, :]
        spl = interp1d(wave, flux, fill_value=0, bounds_error=False)
        combiner.append(spl(new_w))
        hdul.close()
    combiner = np.array(combiner)
    combined = np.median(combiner, axis=0)
    final_out = np.stack([new_w, combined])
    out = targ_name + '_1d_stacked.fits'
    hdr = fits.Header()
    hdr['HIERARCH TARGET NAME'] = targ_name
    hdr['HIERARCH PLX'] = plx
    hdr['HIERARCH RA (deg)'] = ra
    hdr['HIERARCH DEC (deg)'] = dec
    hdr['HIERARCH INST'] = inst
    hdu = fits.PrimaryHDU(final_out, header=hdr)
    try:
        hdu.writeto(out)
    except OSError:
        os.remove(out)
        hdu.writeto(out)
    pass


def merge_echelle(data, header, out='', save=False):
    """Merge echelle orders."""
    waves = np.array([])
    fluxes = np.array([])
    errors = np.array([])
    sn = np.array([])

    next_start = 0

    for i in range(data.shape[1] - 1, 0, -1):
        wave_current = data[0, i, :]
        wave_next = data[0, i - 1, :]
        flux_current = data[1, i, :]
        flux_next = data[1, i - 1, :]
        err_current = data[2, i, :]
        err_next = data[2, i - 1, :]
        sn_current = data[3, i, :]
        sn_next = data[3, i - 1, :]

        spl_current = interp1d(wave_current, sn_current)
        spl_next = interp1d(wave_next, sn_next)

        wav = np.linspace(wave_next[0], wave_current[-1], 1000)

        w = wav[np.argmin(abs(spl_current(wav) - spl_next(wav)))]

        cur_end = np.where(abs(wave_current - w) < 0.1)[0][0]

        wc = wave_current[next_start:cur_end]

        fc = flux_current[next_start:cur_end]

        erc = err_current[next_start:cur_end]

        snc = sn_current[next_start:cur_end]

        waves = np.concatenate((waves, wc))
        fluxes = np.concatenate((fluxes, fc))
        errors = np.concatenate((errors, erc))
        sn = np.concatenate((sn, snc))

        next_start = np.where(abs(wave_next - w) < 0.1)[0][0]

    waves = np.concatenate((waves, wave_next[next_start:]))
    fluxes = np.concatenate((fluxes, flux_next[next_start:]))
    errors = np.concatenate((errors, err_next[next_start:]))
    sn = np.concatenate((sn, sn_next[next_start:]))

    if save:
        prod_out = np.stack([waves, fluxes, errors, sn])

        targ_name = header['HIERARCH TARGET NAME']
        ra = header['HIERARCH RA']
        dec = header['HIERARCH DEC']
        inst = header['INST']

        hdr = fits.Header()
        hdr['HIERARCH TARGET NAME'] = targ_name
        hdr['HIERARCH RA (deg)'] = ra
        hdr['HIERARCH DEC (deg)'] = dec
        hdr['HIERARCH INST'] = inst

        hdu = fits.PrimaryHDU(prod_out, header=hdr)

        if not out:
            out = targ_name
        else:
            date = header['HIERARCH SHUTTER START DATE'].split('-')
            ut = header['HIERARCH SHUTTER START UT'].split(':')
            out += header['HIERARCH TARGET NAME'] + '_'
            for d in date:
                out += d
            out += '_UT'
            for u in ut:
                out += u
        out += '_1d_rest_frame.fits'
        try:
            hdu.writeto(out)
        except OSError:
            os.remove(out)
            hdu.writeto(out)

    return waves, fluxes, errors, sn


def correct_to_rest(data, mask='G2'):
    """Correct the spectra to rest frame and return wavelength and flux."""
    ccf_arr = []
    for o in range(data.shape[1] - 1, 0, -1):
        wave = data[0, o, :]
        flux = data[5, o, :]
        rv_arr, cc = ccf(wave, flux, mask=mask, drv=0.05)
        ccf_arr.append(cc)
    ccf_arr = np.array(ccf_arr)
    med_ccf = np.median(ccf_arr, axis=0)
    _, rv, _, _ = ccf_fit(rv_arr, med_ccf)

    w, f = feros_velocity_correction(
        data[[0, 5, 4, 8], :, :],
        rv * 1000,
        False,
    )

    return w, f



import matplotlib.pyplot as plt
import numpy as np
from scipy.signal import triang
from termcolor import colored
from astropy.io import fits

from .constants import *
from .utils import gauss_fit
from .spectra_utils import correct_to_rest, merge_echelle


def __ensure_png_pdf(png, pdf):
    if not png and not pdf:
        print(
            colored('Warning! No extension selected! Reverting to png!', 'red')
        )
        return False
    return True


def __setup_ticks(ax):
    """Common tick setup"""
    ax.tick_params(
        axis='both', which='major',
        labelsize=tick_labelsize
    )
    for tick in ax.get_yticklabels():
        tick.set_fontname(fontname)
    for tick in ax.get_xticklabels():
        tick.set_fontname(fontname)
    return ax


def ccf_plot(name, rvs, cc):
    """Create a plot of the CCF."""
    f, ax = plt.subplots(figsize=(12, 8))
    ax.plot(rvs, cc, lw=0.7, color='k')
    ax.set_ylabel('CCF',
                  fontsize=fontsize,
                  fontname=fontname
                  )
    ax.set_xlabel('RV (km/s)',
                  fontsize=fontsize,
                  fontname=fontname
                  )
    ax = __setup_ticks(ax)
    plt.savefig(name + '_ccf.pdf', bbox_inches='tight')


def ccf_gauss_plot(name, rvs, cc, amp, rv, sig, off):
    """Create a plot of the CCF with a gaussian fit."""
    f, ax = plt.subplots(figsize=(12, 8))
    ax.scatter(rvs, cc, s=10, marker='.', color='k')
    ax.plot(rvs, gauss_fit(rvs, amp, rv, sig, off))
    ax.set_xlim([rv - 30, rv + 30])
    ax.set_ylabel('CCF',
                  fontsize=fontsize,
                  fontname=fontname
                  )
    ax.set_xlabel('RV (km/s)',
                  fontsize=fontsize,
                  fontname=fontname
                  )
    ax = __setup_ticks(ax)
    plt.savefig(name + '_ccf_fit.pdf', bbox_inches='tight')


def line_plot_from_file(filename, lines, out, name, png=True, pdf=False):
    """Create a plot showing the selected lines.

            Parameters
            ----------
            filename: str
                The file name of the spectrum.
            lines: str, array_like
                A string with the line to plot or an array with the lines to plot.
            out: str
                The output location for the plot.
            name: str
                The name of the object.
            png: bool, optional
                Set to True to output plot in png. Default is True.
            pdf: bool, optional
                Set to True to output plot in pdf. Default is False
            """
    hdul = fits.open(filename)
    data = hdul[0].data

    print('Correcting spectrum to rest frame. (This could take a while)')
    w, f = correct_to_rest(data)
    prod = np.stack((w, f, data[4, :, :], data[8, :, :]))
    print('Merging echelle spectra.')
    waves, fluxes, _, _ = merge_echelle(prod, hdul[0].header)
    line_plot(waves, fluxes, lines, out, name, png=png, pdf=pdf)


def line_plot(waves, fluxes, lines, out, name, png=True, pdf=False):
    """Create a plot showing the selected lines.

        Parameters
        ----------
        waves: array_like
            The wavelength array.
        fluxes: array_like
            The flux array.
        lines: str, array_like
            A string with the line to plot or an array with the lines to plot.
        out: str
            The output location for the plot.
        name: str
            The name of the object.
        png: bool, optional
            Set to True to output plot in png. Default is True.
        pdf: bool, optional
            Set to True to output plot in pdf. Default is False
        """
    if isinstance(lines, str):  # check if lines is a str or a list/array
        lines = [lines]

    png = png if __ensure_png_pdf(png, pdf) else True
    for line in lines:
        print(f'Plotting line {line}')
        f, ax = plt.subplots(figsize=(20, 10))
        ax.plot(waves, fluxes, c='k', lw=.85)

        if line == 'CaHK':
            ax = CaHK_plot(ax)
        elif line == 'Ha':
            ax = Ha_plot(ax)
        elif line == 'HeI':
            ax = HeI_plot(ax)
        elif line == 'NaID1D2':
            ax = NaID1D2_plot(ax)
        if png:
            plt.savefig(f'{out}/{name}_{line}.png', bbox_inches='tight')
        if pdf:
            plt.savefig(f'{out}/{name}_{line}.pdf', bbox_inches='tight')


def CaHK_plot(ax):
    """Plot the CaHK lines and bandpasses."""
    w = triang(1001, True)
    ax.plot(np.linspace(CaK - 1.19, CaK + 1.19, len(w)), w, c='tab:blue', lw=2)
    ax.plot(np.linspace(CaH - 1.19, CaH + 1.19, len(w)), w, c='tab:blue', lw=2)

    ax.axvspan(CaV - 10, CaV + 10, color='tab:blue', alpha=.5)
    ax.axvspan(CaR - 10, CaR + 10, color='tab:blue', alpha=.5)

    ax.axvline(CaK, c='red', ls='dashed')
    ax.axvline(CaH, c='red', ls='dashed')

    ax.set_xlim(3880, 4020)
    ax.set_ylim(0)

    ax.set_title(r'Ca$_{\rm II}$ H+K',
                 fontname=fontname,
                 fontsize=fontsize
                 )
    ax.set_xlabel(r'$\lambda~[\AA]$',
                  fontname=fontname,
                  fontsize=fontsize
                  )
    ax.set_ylabel('Normalized flux',
                  fontname=fontname,
                  fontsize=fontsize
                  )

    return __setup_ticks(ax)


def Ha_plot(ax):
    """Plot the Ha line and bandpass."""
    ax.axvspan(Ha - 0.678, Ha + 0.678, color='tab:blue', alpha=.5)
    ax.axvspan(6545.495, 6556.245, color='tab:blue', alpha=.5)
    ax.axvspan(6575.934, 6584.684, color='tab:blue', alpha=.5)
    ax.axvline(Ha, color='red', ls='dashed')

    ax.set_xlim(6544.495, 6585.684)
    ax.set_ylim(0)

    ax.set_title(r'H$_{\alpha}$',
                 fontname=fontname,
                 fontsize=fontsize
                 )
    ax.set_xlabel(r'$\lambda~[\AA]$',
                  fontname=fontname,
                  fontsize=fontsize
                  )
    ax.set_ylabel('Normalized flux',
                  fontname=fontname,
                  fontsize=fontsize
                  )

    ax.tick_params(
        axis='both', which='major',
        labelsize=tick_labelsize
    )

    return __setup_ticks(ax)


def HeI_plot(ax):
    """Plot the HeI line and bandpass."""
    ax.axvspan(HeI - .2, HeI + .2, color='tab:blue', alpha=.5)
    ax.axvspan(5874.0, 5875.0, color='tab:blue', alpha=.5)
    ax.axvspan(5878.5, 5879.5, color='tab:blue', alpha=.5)

    ax.axvline(HeI, color='red', ls='dashed')

    ax.set_xlim(5873.75, 5879.75)
    ax.set_ylim(0)

    ax.set_title(r'He$_{\rm I}$',
                 fontname=fontname,
                 fontsize=fontsize
                 )
    ax.set_xlabel(r'$\lambda~[\AA]$',
                  fontname=fontname,
                  fontsize=fontsize
                  )
    ax.set_ylabel('Normalized flux',
                  fontname=fontname,
                  fontsize=fontsize
                  )
    return __setup_ticks(ax)


def NaID1D2_plot(ax):
    """Plot the NaID1 and NaID2 lines and bandpasses."""
    ax.axvspan(NaID1 - .5, NaID1 + .5, color='tab:blue', alpha=.5)
    ax.axvspan(NaID2 - .5, NaID2 + .5, color='tab:blue', alpha=.5)
    ax.axvspan(5800, 5810, color='tab:blue', alpha=.5)
    ax.axvspan(6080, 6100, color='tab:blue', alpha=.5)

    ax.axvline(NaID1, color='red', ls='dashed')
    ax.axvline(NaID2, color='red', ls='dashed')

    ax.set_xlim(5790, 6110)
    ax.set_ylim(0)

    ax.set_title(r'Na$_{\rm I}$ D1 D2',
                 fontname=fontname,
                 fontsize=fontsize
                 )
    ax.set_xlabel(r'$\lambda~[\AA]$',
                  fontname=fontname,
                  fontsize=fontsize
                  )
    ax.set_ylabel('Normalized flux',
                  fontname=fontname,
                  fontsize=fontsize
                  )
    return __setup_ticks(ax)


"""Crosscorr.

This module calculates the cross-correlation function of a spectrum with a
binary mask, and fits a gaussian to the resulting CCF.

Author: Jose Vines
"""
import numpy as np
from PyAstronomy import pyasl
from scipy.optimize import curve_fit

from .utils import gauss_fit
from .constants import masksdir


def ccf(wave, flux, mask='G2', rvmin=-300, rvmax=300, drv=0.1):
    """Calculate the cross-correlation function.

    Parameters
    ----------
    wave: array_like
        The wavelength array of the spectrum.
    flux: array_like
        The flux array of the spectrum.
    mask: str, optional
        The binary mask to use. Options are G2, K0, K5, and M2. Default is G2.
    rvmin: float, optional
        The minimum radial velocity to test, in km/s. Default is -300.
    rvmax: float, optional
        The maximum radial velocity to test, in km/s. Default is 300.
    drv: float, optional
        The step in velocity space for the crosscorr algorithm.

    Returns
    -------
    rv_temp: array_like
        The radial velocities tested.
    cc: array_like
        The median normalized CCF.
    """
    # read mask, call crosscorr
    x1 = np.arange(wave[0] - 200, wave[0], wave[1] - wave[0])
    x2 = np.arange(wave[-1], wave[-1] + 200, wave[-1] - wave[-2])
    wtem = np.hstack([x1, wave, x2])

    lines1, lines2, flux_l = np.loadtxt(f'{masksdir}/{mask}.mas', unpack=True)
    lines_idx = np.where((lines1 > wave[0]) & (lines2 < wave[-1]))[0]
    lines1_new = lines1[lines_idx]
    lines2_new = lines2[lines_idx]
    flux_l_new = flux_l[lines_idx]

    ftem = np.zeros(wtem.size)

    for i in range(len(flux_l_new)):
        indices = np.where((wtem >= lines1_new[i]) & (wtem <= lines2_new[i]))
        if indices[0].size > 0:
            ftem[indices[0]] = flux_l_new[i]

    rv_temp, cc = pyasl.crosscorrRV(wave, flux, wtem, ftem, rvmin, rvmax, drv)
    return rv_temp, cc / np.median(cc)


def ccf_fit(rvs, cc):
    """Fit a gaussian to the CCF with curve_fit.

    Parameters
    ----------
    rvs: array_like
        The RV array
    cc: array_like
        The cross-correlation function.
    """
    est = np.argmin(cc)
    rv_min = rvs[est]
    cc_min = cc[est]
    p0 = [cc_min, rv_min, 1., 1.]
    popt, _ = curve_fit(gauss_fit, rvs, cc, p0=p0)
    return popt









#!/usr/bin/env python
# -*- coding: utf-8 -*-

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

with open('README.md', 'r') as fh:
    long_description = fh.read()
    
setup(
    name='cerespp',
    version='0.0.1',
    author='Jose Vines',
    author_email='jose.vines@ug.uchile.cl',
    description='An extension to ceres.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jvines/Ceres-plusplus',
    packages=find_packages(),
    package_data={'cerespp': ['masks']},
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Scientific/Engineering :: Astronomy"
    ],
    python_requires='>=3.6',
    requires=[
        "numpy", "scipy", "matplotlib", "astropy", "PyAstronomy", "tqdm",
        "termcolor"
    ]
)